==> bidibwizard-client-2.0.17-sources.jar: org/bidib/wizard/mvc/main/view/panel/SwitchPortListPanel.java <==
package org.bidib.wizard.mvc.main.view.panel;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JPanel;

import org.apache.commons.collections4.CollectionUtils;
import org.bidib.jbidibc.messages.BidibLibrary;
import org.bidib.jbidibc.messages.Feature;
import org.bidib.jbidibc.messages.utils.ProductUtils;
import org.bidib.wizard.api.locale.Resources;
import org.bidib.wizard.api.model.NodeInterface;
import org.bidib.wizard.api.model.listener.SwitchPortListener;
import org.bidib.wizard.client.common.controller.NodeSelectionProvider;
import org.bidib.wizard.client.common.table.AbstractPortHierarchicalTable;
import org.bidib.wizard.client.common.view.TabPanelProvider;
import org.bidib.wizard.client.common.view.menu.listener.PortListMenuListener;
import org.bidib.wizard.model.ports.SwitchPort;
import org.bidib.wizard.model.ports.event.PortConfigChangeEvent;
import org.bidib.wizard.model.status.SwitchPortStatus;
import org.bidib.wizard.mvc.main.controller.SwitchPortPanelController;
import org.bidib.wizard.mvc.main.model.MainModel;
import org.bidib.wizard.mvc.main.model.SwitchPortTableModel;
import org.bidib.wizard.mvc.main.model.listener.SwitchPortModelListener;
import org.bidib.wizard.mvc.main.view.panel.listener.TabComponentCreator;
import org.bidib.wizard.mvc.main.view.panel.listener.TabVisibilityListener;
import org.bidib.wizard.mvc.main.view.panel.listener.TabVisibilityProvider;
import org.bidib.wizard.mvc.main.view.table.SwitchPortTable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import io.reactivex.rxjava3.subjects.PublishSubject;

public class SwitchPortListPanel
    extends
    SimpleHierarchicalPortListPanel<SwitchPortTableModel, SwitchPortStatus, SwitchPort, SwitchPortListener, SwitchPortModelListener>
    implements TabVisibilityProvider, TabPanelProvider, TabComponentCreator {

    private static final long serialVersionUID = 1L;

    private static final Logger LOGGER = LoggerFactory.getLogger(SwitchPortListPanel.class);

    private final MainModel mainModel;

    private final TabVisibilityListener tabVisibilityListener;

    public SwitchPortListPanel(final SwitchPortPanelController controller, final SwitchPortTableModel tableModel,
        final MainModel mainModel, final TabVisibilityListener tabVisibilityListener,
        final PublishSubject<PortConfigChangeEvent> portConfigChangeEventSubject) {
        super(tableModel, Resources.getString(SwitchPortListPanel.class, "emptyTable"), portConfigChangeEventSubject,
            mainModel);

        this.mainModel = mainModel;
        this.tabVisibilityListener = tabVisibilityListener;
    }

    @Override
    protected AbstractPortHierarchicalTable<SwitchPort> createPortTable(
        SwitchPortTableModel tableModel, String emptyTableText) {

        return new SwitchPortTable(tableModel, emptyTableText);
    }

    @Override
    protected void createTable(
        final SwitchPortTableModel tableModel, String emptyTableText,
        final PublishSubject<PortConfigChangeEvent> portConfigChangeEventSubject,
        final NodeSelectionProvider nodeSelectionProvider) {

        super.createTable(tableModel, emptyTableText, portConfigChangeEventSubject, nodeSelectionProvider);

        // create the portlist menu listener to handle the port menu
        final PortListMenuListener portListMenuListener =
            createPortListMenuListener(this.table, nodeSelectionProvider, null);
        table.setPortListMenuListener(portListMenuListener);
    }

    @Override
    public JPanel getComponent() {
        return this;
    }

    @Override
    public Object getCreator() {
        return this;
    }

    @Override
    public boolean equals(Object other) {
        if (other instanceof TabComponentCreator) {
            TabComponentCreator creator = (TabComponentCreator) other;
            // TODO if more than a single instance is available this must be changed
            if (creator.getCreator() instanceof SwitchPortListPanel) {
                return true;
            }
        }
        return false;
    }

    @Override
    public int hashCode() {
        return super.hashCode();
    }

    @Override
    protected void processListChanged() {
        super.processListChanged();

        boolean hasPortIdentifiers = false;

        List<SwitchPort> ports = new LinkedList<>();
        ports.addAll(getPorts());

        synchronized (ports) {
            for (SwitchPort port : ports) {
                if (port.isRemappingEnabled()) {
                    hasPortIdentifiers = true;
                    break;
                }
            }
        }

        final NodeInterface node = mainModel.getSelectedNode();
        if (node != null) {
            LOGGER.trace("A node is selected.");

            if (ProductUtils.isOneControl(node.getUniqueId()) || ProductUtils.isOneDriveTurn(node.getUniqueId())) {
                LOGGER
                    .info("Set the time ticks factor to 2 for OneControl and OneDriveTurn nodes. Current node: {}",
                        this);
                ((SwitchPortTable) table).setTicksTimeFactor(2);
            }
            else {
                ((SwitchPortTable) table).setTicksTimeFactor(1);
            }

            boolean hasSwitchPortConfigIo = false;
            boolean hasSwitchPortConfigTicks = false;
            boolean hasSwitchPortConfigLoadType = false;

            if (node.getNode().isPortFlatModelAvailable() && CollectionUtils.isNotEmpty(node.getSwitchPorts())) {
                LOGGER.info("Check if at least one switch port has the switch port config available.");
                for (SwitchPort port : node.getSwitchPorts()) {
                    if (!hasSwitchPortConfigIo) {
                        hasSwitchPortConfigIo =
                            port.isPortConfigKeySupported(Byte.valueOf(BidibLibrary.BIDIB_PCFG_SWITCH_CTRL));
                    }
                    if (!hasSwitchPortConfigTicks) {
                        hasSwitchPortConfigTicks =
                            port.isPortConfigKeySupported(Byte.valueOf(BidibLibrary.BIDIB_PCFG_TICKS));
                    }
                    if (!hasSwitchPortConfigLoadType) {
                        hasSwitchPortConfigLoadType =
                            port.isPortConfigKeySupported(Byte.valueOf(BidibLibrary.BIDIB_PCFG_LOAD_TYPE));
                    }

                    if (hasSwitchPortConfigIo && hasSwitchPortConfigTicks && hasSwitchPortConfigLoadType) {
                        // found one -> show the column
                        break;
                    }
                }
            }
            else {
                Feature switchPortConfigAvailable =
                    Feature.findFeature(node.getNode().getFeatures(), BidibLibrary.FEATURE_SWITCH_CONFIG_AVAILABLE);
                if (switchPortConfigAvailable != null) {
                    hasSwitchPortConfigIo = (switchPortConfigAvailable.getValue() > 0);
                    hasSwitchPortConfigTicks = hasSwitchPortConfigIo;
                }

                for (SwitchPort port : node.getSwitchPorts()) {
                    if (!hasSwitchPortConfigIo) {
                        hasSwitchPortConfigIo =
                            port.isPortConfigKeySupported(Byte.valueOf(BidibLibrary.BIDIB_PCFG_SWITCH_CTRL));
                    }
                    if (!hasSwitchPortConfigTicks) {
                        hasSwitchPortConfigTicks =
                            port.isPortConfigKeySupported(Byte.valueOf(BidibLibrary.BIDIB_PCFG_TICKS));
                    }
                    if (!hasSwitchPortConfigLoadType) {
                        hasSwitchPortConfigLoadType =
                            port.isPortConfigKeySupported(Byte.valueOf(BidibLibrary.BIDIB_PCFG_LOAD_TYPE));
                    }

                    if (hasSwitchPortConfigIo && hasSwitchPortConfigTicks && hasSwitchPortConfigLoadType) {
                        // found one -> show the column
                        break;
                    }
                }
            }
            LOGGER
                .info(
                    "List has changed, hasPortIdentifiers: {}, hasSwitchPortConfigIo: {}, hasSwitchPortConfigTicks: {}",
                    hasPortIdentifiers, hasSwitchPortConfigIo, hasSwitchPortConfigTicks);

            // keep the column index of the column to insert in the view
            int viewColumnIndex = SwitchPortTableModel.COLUMN_IO_BEHAVIOUR;

            // show/hide the IO behaviour column
            viewColumnIndex =
                table
                    .setColumnVisible(SwitchPortTableModel.COLUMN_IO_BEHAVIOUR, viewColumnIndex, hasSwitchPortConfigIo);

            // show/hide the ticks column
            viewColumnIndex =
                table
                    .setColumnVisible(SwitchPortTableModel.COLUMN_SWITCH_OFF_TIME, viewColumnIndex,
                        hasSwitchPortConfigTicks);

            // show/hide the load type column
            viewColumnIndex =
                table
                    .setColumnVisible(SwitchPortTableModel.COLUMN_LOAD_TYPE, viewColumnIndex,
                        hasSwitchPortConfigLoadType);

            // show/hide the port identifier column
            viewColumnIndex =
                table
                    .setColumnVisible(SwitchPortTableModel.COLUMN_PORT_IDENTIFIER, viewColumnIndex, hasPortIdentifiers);
        }
        else {
            ((SwitchPortTable) table).setTicksTimeFactor(1);
        }

        tabVisibilityListener.setTabVisible(this, isTabVisible());
    }

    @Override
    public Class<?> getPortClass() {
        return SwitchPort.class;
    }

    @Override
    public boolean isTabVisible() {
        final NodeInterface node = mainModel.getSelectedNode();
        if (node != null) {
            boolean isTabVisible = node.hasSwitchPorts();
            LOGGER.debug("Check if tab is visible: {}", isTabVisible);
            return isTabVisible;
        }
        return false;
    }

    @Override
    protected List<SwitchPort> getPorts() {
        final NodeInterface node = mainModel.getSelectedNode();
        if (node != null) {
            List<SwitchPort> ports = new ArrayList<>();
            ports.addAll(node.getSwitchPorts());

            return ports;
        }
        return Collections.emptyList();
    }

    public void refreshView() {
        listChanged();
    }
}

==> bidibwizard-client-2.0.29-sources.jar: org/bidib/wizard/mvc/main/view/panel/SwitchPortListPanel.java <==
package org.bidib.wizard.mvc.main.view.panel;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.JPanel;

import org.apache.commons.collections4.CollectionUtils;
import org.bidib.jbidibc.messages.BidibLibrary;
import org.bidib.jbidibc.messages.Feature;
import org.bidib.jbidibc.messages.utils.ProductUtils;
import org.bidib.wizard.api.locale.Resources;
import org.bidib.wizard.api.model.NodeInterface;
import org.bidib.wizard.api.model.listener.SwitchPortListener;
import org.bidib.wizard.client.common.controller.NodeSelectionProvider;
import org.bidib.wizard.client.common.model.listener.PortTableLayoutListener;
import org.bidib.wizard.client.common.table.AbstractPortHierarchicalTable;
import org.bidib.wizard.client.common.view.TabPanelProvider;
import org.bidib.wizard.client.common.view.menu.listener.PortListMenuListener;
import org.bidib.wizard.model.ports.SwitchPort;
import org.bidib.wizard.model.ports.event.PortConfigChangeEvent;
import org.bidib.wizard.model.status.SwitchPortStatus;
import org.bidib.wizard.mvc.main.controller.SwitchPortPanelController;
import org.bidib.wizard.mvc.main.model.MainModel;
import org.bidib.wizard.mvc.main.model.SwitchPortTableModel;
import org.bidib.wizard.mvc.main.model.listener.SwitchPortModelListener;
import org.bidib.wizard.mvc.main.view.panel.listener.TabComponentCreator;
import org.bidib.wizard.mvc.main.view.panel.listener.TabVisibilityListener;
import org.bidib.wizard.mvc.main.view.panel.listener.TabVisibilityProvider;
import org.bidib.wizard.mvc.main.view.table.SwitchPortTable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import io.reactivex.rxjava3.subjects.PublishSubject;

public class SwitchPortListPanel
    extends
    SimpleHierarchicalPortListPanel<SwitchPortTableModel, SwitchPortStatus, SwitchPort, SwitchPortListener, SwitchPortModelListener>
    implements TabVisibilityProvider, PortTableLayoutListener, TabPanelProvider, TabComponentCreator {

    private static final long serialVersionUID = 1L;

    private static final Logger LOGGER = LoggerFactory.getLogger(SwitchPortListPanel.class);

    private final MainModel mainModel;

    private final TabVisibilityListener tabVisibilityListener;

    public SwitchPortListPanel(final SwitchPortPanelController controller, final SwitchPortTableModel tableModel,
        final MainModel mainModel, final TabVisibilityListener tabVisibilityListener,
        final PublishSubject<PortConfigChangeEvent> portConfigChangeEventSubject) {
        super(tableModel, Resources.getString(SwitchPortListPanel.class, "emptyTable"), portConfigChangeEventSubject,
            mainModel);

        this.mainModel = mainModel;
        this.tabVisibilityListener = tabVisibilityListener;
    }

    @Override
    protected AbstractPortHierarchicalTable<SwitchPort> createPortTable(
        SwitchPortTableModel tableModel, String emptyTableText) {

        return new SwitchPortTable(tableModel, emptyTableText);
    }

    @Override
    protected void createTable(
        final SwitchPortTableModel tableModel, String emptyTableText,
        final PublishSubject<PortConfigChangeEvent> portConfigChangeEventSubject,
        final NodeSelectionProvider nodeSelectionProvider) {

        super.createTable(tableModel, emptyTableText, portConfigChangeEventSubject, nodeSelectionProvider);

        // create the portlist menu listener to handle the port menu
        final PortListMenuListener portListMenuListener =
            createPortListMenuListener(this.table, nodeSelectionProvider, null);
        table.setPortListMenuListener(portListMenuListener);
    }

    @Override
    public JPanel getComponent() {
        return this;
    }

    @Override
    public Object getCreator() {
        return this;
    }

    @Override
    public boolean equals(Object other) {
        if (other instanceof TabComponentCreator) {
            TabComponentCreator creator = (TabComponentCreator) other;
            // TODO if more than a single instance is available this must be changed
            if (creator.getCreator() instanceof SwitchPortListPanel) {
                return true;
            }
        }
        return false;
    }

    @Override
    public int hashCode() {
        return super.hashCode();
    }

    @Override
    protected void processListChanged() {
        super.processListChanged();

        final NodeInterface node = mainModel.getSelectedNode();
        if (node != null) {
            LOGGER.trace("A node is selected.");

            if (ProductUtils.isOneControl(node.getUniqueId()) || ProductUtils.isOneDriveTurn(node.getUniqueId())) {
                LOGGER
                    .info("Set the time ticks factor to 2 for OneControl and OneDriveTurn nodes. Current node: {}",
                        this);
                ((SwitchPortTable) table).setTicksTimeFactor(2);
            }
            else {
                ((SwitchPortTable) table).setTicksTimeFactor(1);
            }
            portTableLayoutChanged(node);
        }
        else {
            ((SwitchPortTable) table).setTicksTimeFactor(1);
        }

        tabVisibilityListener.setTabVisible(this, isTabVisible());
    }

    @Override
    public void portTableLayoutChanged(final NodeInterface node) {
        boolean hasPortIdentifiers = false;

        final List<SwitchPort> ports = new ArrayList<>();
        ports.addAll(getPorts());

        synchronized (ports) {
            for (SwitchPort port : ports) {
                if (port.isRemappingEnabled()) {
                    hasPortIdentifiers = true;
                    break;
                }
            }
        }

        boolean hasSwitchPortConfigIo = false;
        boolean hasSwitchPortConfigTicks = false;
        boolean hasSwitchPortConfigLoadType = false;

        if (node.getNode().isPortFlatModelAvailable() && CollectionUtils.isNotEmpty(node.getSwitchPorts())) {
            LOGGER.info("Check if at least one switch port has the switch port config available.");
            for (SwitchPort port : node.getSwitchPorts()) {
                if (!hasSwitchPortConfigIo) {
                    hasSwitchPortConfigIo =
                        port.isPortConfigKeySupported(Byte.valueOf(BidibLibrary.BIDIB_PCFG_SWITCH_CTRL));
                }
                if (!hasSwitchPortConfigTicks) {
                    hasSwitchPortConfigTicks =
                        port.isPortConfigKeySupported(Byte.valueOf(BidibLibrary.BIDIB_PCFG_TICKS));
                }
                if (!hasSwitchPortConfigLoadType) {
                    hasSwitchPortConfigLoadType =
                        port.isPortConfigKeySupported(Byte.valueOf(BidibLibrary.BIDIB_PCFG_LOAD_TYPE));
                }

                if (hasSwitchPortConfigIo && hasSwitchPortConfigTicks && hasSwitchPortConfigLoadType) {
                    // found one -> show the column
                    break;
                }
            }
        }
        else {
            Feature switchPortConfigAvailable =
                Feature.findFeature(node.getNode().getFeatures(), BidibLibrary.FEATURE_SWITCH_CONFIG_AVAILABLE);
            if (switchPortConfigAvailable != null) {
                hasSwitchPortConfigIo = (switchPortConfigAvailable.getValue() > 0);
                hasSwitchPortConfigTicks = hasSwitchPortConfigIo;
            }

            for (SwitchPort port : node.getSwitchPorts()) {
                if (!hasSwitchPortConfigIo) {
                    hasSwitchPortConfigIo =
                        port.isPortConfigKeySupported(Byte.valueOf(BidibLibrary.BIDIB_PCFG_SWITCH_CTRL));
                }
                if (!hasSwitchPortConfigTicks) {
                    hasSwitchPortConfigTicks =
                        port.isPortConfigKeySupported(Byte.valueOf(BidibLibrary.BIDIB_PCFG_TICKS));
                }
                if (!hasSwitchPortConfigLoadType) {
                    hasSwitchPortConfigLoadType =
                        port.isPortConfigKeySupported(Byte.valueOf(BidibLibrary.BIDIB_PCFG_LOAD_TYPE));
                }

                if (hasSwitchPortConfigIo && hasSwitchPortConfigTicks && hasSwitchPortConfigLoadType) {
                    // found one -> show the column
                    break;
                }
            }
        }
        LOGGER
            .info("List has changed, hasPortIdentifiers: {}, hasSwitchPortConfigIo: {}, hasSwitchPortConfigTicks: {}",
                hasPortIdentifiers, hasSwitchPortConfigIo, hasSwitchPortConfigTicks);

        // keep the column index of the column to insert in the view
        int viewColumnIndex = SwitchPortTableModel.COLUMN_IO_BEHAVIOUR;

        // show/hide the IO behaviour column
        viewColumnIndex =
            table.setColumnVisible(SwitchPortTableModel.COLUMN_IO_BEHAVIOUR, viewColumnIndex, hasSwitchPortConfigIo);

        // show/hide the ticks column
        viewColumnIndex =
            table
                .setColumnVisible(SwitchPortTableModel.COLUMN_SWITCH_OFF_TIME, viewColumnIndex,
                    hasSwitchPortConfigTicks);

        // show/hide the load type column
        viewColumnIndex =
            table.setColumnVisible(SwitchPortTableModel.COLUMN_LOAD_TYPE, viewColumnIndex, hasSwitchPortConfigLoadType);

        // show/hide the port identifier column
        viewColumnIndex =
            table.setColumnVisible(SwitchPortTableModel.COLUMN_PORT_IDENTIFIER, viewColumnIndex, hasPortIdentifiers);
    }

    @Override
    public Class<?> getPortClass() {
        return SwitchPort.class;
    }

    @Override
    public boolean isTabVisible() {
        final NodeInterface node = mainModel.getSelectedNode();
        if (node != null) {
            boolean isTabVisible = node.hasSwitchPorts();
            LOGGER.debug("Check if tab is visible: {}", isTabVisible);
            return isTabVisible;
        }
        return false;
    }

    @Override
    protected List<SwitchPort> getPorts() {
        final NodeInterface node = mainModel.getSelectedNode();
        if (node != null) {
            List<SwitchPort> ports = new ArrayList<>();
            ports.addAll(node.getSwitchPorts());

            return ports;
        }
        return Collections.emptyList();
    }

    public void refreshView() {
        listChanged();
    }
}
